/*******************************************************************
 *
 *	File:		Controls.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 200s, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __CONTROLS_H
#define __CONTROLS_H

#include <coecntrl.h>
#include <eikcmbut.h>


/*****************************************************
 *
 * CBitmappedCoeControl:
 * - allows the control to draw to an offscreen bitmap
 *   which get blitted to screen
 * - control should call DrawControl(); when it wants
 *   to redraw itself.
 * - CBitmappedCoeControl will gracefully cope with the fact
 *   that the offline bitmap cannot be allocated due to
 *   lack of memory
 *
 *****************************************************/

class CBitmappedCoeControl : public CCoeControl
{
public:
	~CBitmappedCoeControl() {DestroyBitmap();}
	void DrawControl();
protected:
	virtual void DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& aBitmapDevice) const = 0;
	void SizeChanged();
	void DestroyBitmap();
	void CreateBitmapL(const TSize& aSize);
	void Draw(const TRect& aRect) const;
protected:
	CFbsBitGc*			iBitmapContext;
	CFbsBitmap*			iBitmap;
	CFbsBitmapDevice*	iBitmapDevice;
};



class CProgressControl : public CBitmappedCoeControl
{
public:
	CProgressControl();
	void SetProgress(TInt aPercent, TBool aDrawNow=ETrue);

protected:
	CCoeControl* ComponentControl(TInt /*aIndex*/) const{return NULL;}
	TInt CountComponentControls() const {return 0;}
	virtual void DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& aBitmapDevice) const;
	TRgb	iProgressColor;
	TInt	iProgressPercent;	// in percent
};


#endif			/* __CONTROLS_H */

