/*******************************************************************
 *
 *	File:		Dialogs.h
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __DIALOGS_H
#define __DIALOGS_H

#include <eikenv.h>
#include <eikdialg.h>
#include <eiktxlbx.h>
#include <eiktxlbm.h>
#include <eikdlgtb.h>
#include <eiklabel.h>
#include <eikchlst.h>
#include <eikchkbx.h>
#include <eikedwob.h>

#include <emame.rsg>

class CMameEngine;
class CProgressControl;
class TGameOptions;
class CInstalledGameList;
class CRichText;
class CEikRichTextEditor;



/************************************************
 *
 * GameRunning Dialog
 *
 ************************************************/


class CGameRunningDialog : public CEikDialog
{
public:
	CGameRunningDialog(const TDesC8& aGameName, CMameEngine& aMameEngine);
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
	void HandleControlStateChangeL(TInt aControlId);

	TBuf<64>			iGameName;
	CMameEngine&		iMameEngine;
};


/************************************************
 *
 * Scanning ROMs Dialog
 *
 ************************************************/

class CScanningRomDialog : public CEikDialog
{
public:
	CScanningRomDialog(CMameEngine& aMameEngine, CInstalledGameList& aGameList);
	~CScanningRomDialog();
protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();
//	void SetSizeAndPositionL(const TSize& aSize);
	SEikControlInfo CreateCustomControlL(TInt aControlType);
protected:
	static TInt IdleCallBack(TAny*);
	TInt DoUpdate();
	void UpdateProgress();

	TInt					iGameNo;
	TInt					iNoOfGames;
	CMameEngine&			iMameEngine;
	CIdle*					iIdle;
	CProgressControl*		iProgress;
	CInstalledGameList&		iInstalledGameList;
};


/************************************************
 *
 * Game Info Dialog
 *
 ************************************************/
#if 0
class CGameInfoDialog : public CEikDialog, public MEikEdwinSizeObserver
{
public:
	CGameInfoDialog(CMameEngine& aMameEngine, TInt aGame) : iMameEngine(aMameEngine), iGame(aGame){}

public:			// implements MEikEdwinSizeObserver
	virtual TBool HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize);

protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();
	void SSizeAndPositionL(const TSize& aSize);
	void ShowTextL(CRichText& aRichText);
	void AppendTextL(TDes& aOutput, TInt aResId, const TDesC& aGameText);


	CMameEngine&		iMameEngine;
	TInt				iGame;
};

#endif
/************************************************
 *
 * Game Options Dialog
 *
 ************************************************/


class CGameOptionsDialog : public CEikDialog
{
public:
	CGameOptionsDialog(TGameOptions& aGameOptions) : iGameOptions(aGameOptions){}
protected: // framework
    void PreLayoutDynInitL();
	void SetSizeAndPositionL(const TSize& aSize);
    TBool OkToExitL(TInt aKeycode);
	TGameOptions&		iGameOptions;
};




/************************************************
 *
 * Play List Dialog
 *
 ************************************************/

class CPlayListDialog : public CEikDialog
{
public:
	CPlayListDialog(CMameEngine& aMameEngine, TInt& aGameToRun, TInt aTitleResId) 
	: iMameEngine(aMameEngine), iGameToRun(aGameToRun), iTitleResId(aTitleResId){}
protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();

	void HandleControlStateChangeL(TInt aControlId);
	void SetSizeAndPositionL(const TSize& aSize);
	void ShowGameDetailsL(TInt aGame);
	TInt GameIndex();

	virtual TInt NoOfGames() = 0;
	virtual TInt GameNo(TInt aIndex) = 0;

    TBool OkToExitL(TInt aKeycode);
	CMameEngine&			iMameEngine;
	TInt&					iGameToRun;
	TInt					iTitleResId;
};

/************************************************
 *
 * All games play List Dialog
 *
 ************************************************/

class CAllGamesPlayListDialog : public CPlayListDialog
{
public:
	CAllGamesPlayListDialog(CMameEngine& aMameEngine, TInt& aGameToRun) :
		CPlayListDialog(aMameEngine, aGameToRun, R_TEXT_PLAY_LIST_ALL_GAMES){}
protected:			// from CPlayListDialog
	virtual TInt NoOfGames();
	virtual TInt GameNo(TInt aIndex);
};

/************************************************
 *
 * Installed games play List Dialog
 *
 ************************************************/

class CInstalledGamesPlayListDialog : public CPlayListDialog
{
public:
	CInstalledGamesPlayListDialog(CMameEngine& aMameEngine, TInt& aGameToRun) :
		CPlayListDialog(aMameEngine, aGameToRun, R_TEXT_PLAY_LIST_INSTALLED_GAMES){}
protected:			// from CPlayListDialog
	virtual TInt NoOfGames();
	virtual TInt GameNo(TInt aIndex);
};


/************************************************
 *
 * CSimpleTextInfo Dialog
 *
 ************************************************/

class CSimpleTextInfoDialog : public CEikDialog, public MEikEdwinSizeObserver
{
public:
	CSimpleTextInfoDialog(TInt aTextIdOne = -1, TInt aRichTextCtlIdOne = -1, 
		TInt aTextIdTwo = -1, TInt aRichTextCtlIdTwo = -1,
		TBool aSimpleTextResIdOneIsArray = EFalse, TBool aSimpleTextResIdTwoIsArray = EFalse
		);
	void SetDialogBackground(TBool aEnable){iSetDialogBackground=aEnable;}
	void WantVertScrollbar(TBool aEnable){iWantVertScrollbar=aEnable;}
public:			// implements MEikEdwinSizeObserver
	virtual TBool HandleEdwinSizeEventL(CEikEdwin* aEdwin, TEdwinSizeEvent aEventType, TSize aDesirableEdwinSize);

protected: // framework
    void PreLayoutDynInitL();
	void PostLayoutDynInitL();

protected:	// new stuff
	virtual void ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId);
	virtual void PreLayoutDynInitRichTextL(CEikRichTextEditor& aRichTextEditor, TInt aRichTextCtlId, TInt aResId);

	void ShowSimpleTextL(const TDesC& aSimpleText, CRichText& aRichText);

	TInt	iSimpleTextResIdOne;
	TInt	iSimpleTextResIdTwo;
	TInt	iRichTextCtlIdOne;
	TInt	iRichTextCtlIdTwo;
	TBool	iSimpleTextResIdOneIsArray;
	TBool	iSimpleTextResIdTwoIsArray;
	TBool	iSetDialogBackground;
	TBool	iWantVertScrollbar;
};




/************************************************
 *
 * Game Info Dialog
 *
 ************************************************/

class CGameInfoDialog : public CSimpleTextInfoDialog
{
public:
	CGameInfoDialog(CMameEngine& aMameEngine, TInt aGame);
protected:	// from CSimpleTextInfoDialog
	virtual void ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId);

protected:	// new stuff
	void AppendTextL(TDes& aOutput, TInt aResId, const TDesC& aGameText);
	CMameEngine&		iMameEngine;
	TInt				iGame;
};

/************************************************
 *
 * About Dialog
 *
 ************************************************/

class CAboutDialog : public CSimpleTextInfoDialog
{
public:
	CAboutDialog();
protected:	// from CSimpleTextInfoDialog
	virtual void ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId);
};



/************************************************
 *
 * Game has no rom Dialog
 *
 ************************************************/

class CGameHasNoRomDialog : public CSimpleTextInfoDialog
{
public:
	CGameHasNoRomDialog(const TDesC& aGameName, const TDesC& aGameShortName);
protected:	// from CSimpleTextInfoDialog
	virtual void ShowTextL(CRichText& aRichText, TInt aRichTextCtlId, TInt aResId);
	TBuf<64>	iGameName;
	TBuf<32>	iGameShortName;
};


#endif			/* __DIALOGS_H */