/*******************************************************************
 *
 *	File:		EMameApp.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#include "dialogs.h"
#include "emameapp.h"
#include "version.h"

#include <emame.rsg>
#include <emame.mbg>

extern "C" {
#include "audit.h"
}

_LIT(KLastGameFile, "c:\\system\\apps\\emame\\lastgame.ini");

CApaDocument* CMameApplication::CreateDocumentL()
{
	CMameDocument*	doc = new (ELeave) CMameDocument(*this);

	CleanupStack::PushL(doc);
	doc->ConstructL();
	CleanupStack::Pop();

	return doc;
}



CMameDocument::CMameDocument(CEikApplication& aApp)
		: CEikDocument(aApp)
{
}

CMameDocument::~CMameDocument()
{
	delete iMameEngine;
}

void CMameDocument::ConstructL()
{
	iMameEngine = CMameEngine::NewL(CEikonEnv::Static()->FsSession());
}


CEikAppUi* CMameDocument::CreateAppUiL()
{
	return new(ELeave) CMameAppUi;
}


CMameAppUi::~CMameAppUi()
{
	delete iAppView;
	delete iGameRunner;
}

void CMameAppUi::ConstructL()
{
	BaseConstructL();
	iAppView=new(ELeave) CMameAppView;
	iAppView->ConstructL(ClientRect());

	TRAPD(err, EnableAntiAliasingL());

	MameEngine().SetWindowGroupId(CEikonEnv::Static()->RootWin().Identifier());

	LoadLastRunningGame();

	iGameRunner = CGameRunner::NewL(*this, MameEngine());

	iLastDialogIsInstalledGames = ETrue;
	DisplayAllGamesOrInstalledGamesDialogL();
}


void CMameAppUi::EnableAntiAliasingL()
{
	CFbsTypefaceStore* store = CFbsTypefaceStore::NewL(CEikonEnv::Static()->ScreenDevice());
	store->SetDefaultBitmapType(EAntiAliasedGlyphBitmap);
	delete store;
}


void CMameAppUi::HandleApplicationSpecificEventL(TInt aType, const TWsEvent& aEvent)
{
	switch (aType)
	{
		case EMameShuttingDown:
			CEikonEnv::Static()->RootWin().SetOrdinalPosition(0);
//			MameEngine().ShutdownGame();
			StopGame();
//			DisplayAllGamesOrInstalledGamesDialogL();		// put up dialog again
			break;

		default:
			break;
	}
}


void CMameAppUi::DisplayHelpDialogL()
{
	CSimpleTextInfoDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_EMAME_HELP, 1, -1, -1, ETrue);
	dialog->ExecuteLD(R_DIALOG_DIPLAY_EMAME_HELP);
}


void CMameAppUi::HandleCommandL(TInt aCommand)
{
	switch (aCommand)
	{
		case EEikCmdExit: 
			Exit();
			break;

		case EEikCmdGameAllGames: 
			DisplayAllGamesPlayDialogL();
			break;

		case EEikCmdGameInstalledGames: 
			DisplayInstalledGamesPlayDialogL();
			break;

		case EEikCmdGameScanGames:
			ScanInstalledGamesL();
			if (MameEngine().InstalledGameList().NoOfGames() > 0)
				DisplayInstalledGamesPlayDialogL();
			break;

		case EEikCmdGameOptions:
			DisplayGameOptionsDialogL();
			break;

		case EEikCmdHelpPlayEmame:
			DisplayHelpDialogL();
			break;

		case EEikCmdHelpAbout:
			DisplayAboutDialogL();
			break;
	}
}




void CMameAppUi::DisplayGameOptionsDialogL()
{
	TGameOptions	gameOptions = MameEngine().GameOptions();
	CEikDialog* dialog = new(ELeave) CGameOptionsDialog(gameOptions);

	if (dialog->ExecuteLD(R_DIALOG_GAME_OPTIONS))
	{
		MameEngine().SetGameOptions(gameOptions);
	}
}

TBool CMameAppUi::ScanInstalledGamesL()
{
	MameEngine().CreateZipList();

	CInstalledGameList*	list = CInstalledGameList::NewL(MameEngine().NoOfGames());
	CleanupStack::PushL(list);
	CEikDialog* dialog = new(ELeave) CScanningRomDialog(MameEngine(), *list);
	TBool ok = dialog->ExecuteLD(R_DIALOG_DIPLAY_EMAME_SCANNING) == EEikBidOk;
	if (ok)
		MameEngine().SetInstalledGameList(*list);
	CleanupStack::PopAndDestroy(list);

	const CInstalledGameList&	engineList = MameEngine().InstalledGameList();
	if (engineList.NoOfGames() == 0)
	{
		CEikDialog* dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_NO_ROMS_FOUND, 1);
		dialog->ExecuteLD(R_DIALOG_NO_ROMS_FOUND);
	}
	MameEngine().DestroyZipList();
	return ok;
}


void CMameAppUi::DisplayInstalledGamesPlayDialogL()
{
	if (MameEngine().InstalledGameList().NoOfGames() == 0)
	{
		CEikDialog*	dialog = new(ELeave) CSimpleTextInfoDialog(R_SIMPLE_TEXT_NO_INSTALLED_GAMES, 1);
		if (dialog->ExecuteLD(R_DIALOG_NO_INSTALLED_GAMES))
			ScanInstalledGamesL();
	}

	if (MameEngine().InstalledGameList().NoOfGames() > 0)
	{
		CEikDialog*	dialog = new(ELeave) CInstalledGamesPlayListDialog(MameEngine(), iGameToRun);
		if (dialog->ExecuteLD(R_DIALOG_PLAY_LIST))
		{
			RunGameL();
			iLastDialogIsInstalledGames = ETrue;
		}
	}
}


void CMameAppUi::DisplayAllGamesOrInstalledGamesDialogL()
{
	if (iLastDialogIsInstalledGames)
		DisplayInstalledGamesPlayDialogL();
	else
		DisplayAllGamesPlayDialogL();
}


void CMameAppUi::DisplayAllGamesPlayDialogL()
{
	CEikDialog*	dialog = new(ELeave) CAllGamesPlayListDialog(MameEngine(), iGameToRun);
	if (dialog->ExecuteLD(R_DIALOG_PLAY_LIST))
	{
		RunGameL();
		iLastDialogIsInstalledGames = EFalse;
	}
}

void CMameAppUi::LoadLastRunningGame()
{
	TPckg<TInt>	pkg(iGameToRun);
	RFile	file;
	TInt err = file.Open(iCoeEnv->FsSession(), KLastGameFile, 0);
	if (!err)
	{
		file.Read(pkg);
		file.Close();
	}
	else
		iGameToRun = -1;
}

void CMameAppUi::SaveLastRunningGame()
{
	TPckgC<TInt>	pkg(iGameToRun);
	RFile	file;
	TInt err = file.Open(iCoeEnv->FsSession(), KLastGameFile, EFileWrite);
	if (err)
		err = file.Create(iCoeEnv->FsSession(), KLastGameFile, EFileWrite);
	if (!err)
	{
		file.Write(pkg);
		file.Close();
	}
}


void CMameAppUi::RunGameL()
{
	TInt missing = RomsetMissing(iGameToRun);
	if (missing)
	{
		TBuf<64>	gameName;
		TPtrC8		gameName8((TUint8*)MameEngine().GameName(iGameToRun));
		gameName.Copy(gameName8);

		TBuf<32>	gameShortName;
		TPtrC8		gameShortName8((TUint8*)MameEngine().GameShortName(iGameToRun));
		gameShortName.Copy(gameShortName8);

		CEikDialog*	dialog = new(ELeave) CGameHasNoRomDialog(gameName, gameShortName);
		dialog->ExecuteLD(R_DIALOG_GAME_HAS_NO_ROM);
	}
	else
	{
		TBuf8<64>	gameName;
		iGameRunner->RunGameL(iGameToRun);
	
		gameName.Format(_L8("%s"), MameEngine().GameName(iGameToRun));
		SaveLastRunningGame();
		iGameRunningDialog = new(ELeave) CGameRunningDialog(gameName, MameEngine());
		iGameRunningDialog->ExecuteLD(R_DIALOG_RUNNING_GAME);
	
		DisplayAllGamesOrInstalledGamesDialogL();
	}
}


void CMameAppUi::StopGame()
{
	iGameRunner->TerminateGame();
	if (iGameRunningDialog)
	{
		delete iGameRunningDialog;
		iGameRunningDialog = NULL;
	}
}

void CMameAppUi::NotifyGameFinished()
{
	StopGame();
}


void CMameAppUi::DisplayAboutDialogL()
{
	CEikDialog*	dialog = new(ELeave) CAboutDialog;
	dialog->ExecuteLD(R_DIALOG_ABOUT);
}



void CMameAppView::ConstructL(const TRect& aRect)
{
	CreateWindowL();
	SetRect(aRect);
	ActivateL();

		/*
		 * Load background image
		 */
	TBuf<1> name = _L("*");
	TRAPD(err, iBgImage = CEikonEnv::Static()->CreateBitmapL(name, EMbmEmameEmame256));
	if (iBgImage)
	{
		iImagePosition.iX = (aRect.Size().iWidth - iBgImage->SizeInPixels().iWidth) / 2;
		iImagePosition.iY = (aRect.Size().iHeight - iBgImage->SizeInPixels().iHeight) / 2;
	}
}

CMameAppView::~CMameAppView()
{
	delete iBgImage;
}


void CMameAppView::Draw(const TRect& /*aRect*/) const
{
	CWindowGc& gc = SystemGc();

	if (iBgImage)
		gc.DrawBitmap(iImagePosition, iBgImage);
}


GLDEF_C TInt E32Dll(TDllReason)
{
	return KErrNone;
}

EXPORT_C CApaApplication* NewApplication()
{
	return new CMameApplication;
}



TUid CMameApplication::AppDllUid() const
{
	return KUidEmameApp;
}

