/*******************************************************************
 *
 *	File:		EMameApp.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __EMAME_APP_H
#define __EMAME_APP_H

#include <coecntrl.h>
#include <coeccntx.h>
#include <coemain.h>

#include <eikappui.h>
#include <eikapp.h>
#include <eikdoc.h>

#include "emame.hrh"

#include "EmameEngine.h"


const TUid KUidEmameApp = { 0x10009BB3 };


class CWsBitmap;

class CMameDocument : public CEikDocument
{
public:
	~CMameDocument();
	CMameDocument(CEikApplication& aApp);
	void ConstructL();

	CMameEngine&	MameEngine(){return *iMameEngine;}

protected:
	CMameEngine*	iMameEngine;

private: // from CEikDocument
	CEikAppUi* CreateAppUiL();
};


class CMameAppView : public CCoeControl, public MCoeControlBrushContext
{
public:
	~CMameAppView();
	void ConstructL(const TRect& aRect);
private:
	void Draw(const TRect& /*aRect*/) const;

	CWsBitmap*	iBgImage;
	TPoint		iImagePosition;
};


class CMameAppUi : public CEikAppUi, MGameWatcher
{
public:
	CMameAppUi() : iGameToRun(-1){}
	void ConstructL();
	~CMameAppUi();

		// cannot be called during construction of the document!
	static CMameAppUi*	Static() {return (CMameAppUi*) CEikonEnv::Static()->AppUi();}

protected: // from CEikAppUi
	void EnableAntiAliasingL();

	virtual void HandleApplicationSpecificEventL(TInt aType,const TWsEvent& aEvent);

	void HandleCommandL(TInt aCommand);
	void DisplayAllGamesPlayDialogL();
	void DisplayInstalledGamesPlayDialogL();
	void DisplayGameOptionsDialogL();
	void DisplayHelpDialogL();
	void DisplayAboutDialogL();

	TBool ScanInstalledGamesL();

	void LoadLastRunningGame();
	void SaveLastRunningGame();

	TFileName		iLastGameFileName;

	CMameEngine& MameEngine(){return ((CMameDocument*) Document())->MameEngine();}
	CGameRunner*	iGameRunner;
	CEikDialog*		iGameRunningDialog;

	void StartGameL(TInt aGame);
	void StopGame();

	void DisplayAllGamesOrInstalledGamesDialogL();
	TBool	iLastDialogIsInstalledGames;

protected:
	virtual void NotifyGameFinished();

protected:
	TInt		iGameToRun;

	void RunGameL();

private:
	CMameAppView* iAppView;
};






class CMameApplication : public CEikApplication
	{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	};

#endif
