/*******************************************************************
 *
 *	File:		EMameEngine.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#ifndef __EMAME_ENGINE_H
#define __EMAME_ENGINE_H

#include <e32std.h>
#include <e32base.h>

#include "EMame.h"
#include "EMameGlobal.h"


class CInstalledGameList : public CBase
{
public:
	static CInstalledGameList* NewL(TInt aMaxGames);
	CInstalledGameList(const CInstalledGameList& aOther);
	CInstalledGameList& operator=(const CInstalledGameList& aOther);
	~CInstalledGameList();

	TInt NoOfGames() const {return iGames;}
	TInt GameNo(TInt aGameIndex) const;
	void AddGame(TInt aGameNo);
	TInt Externalise(RFile& aFile);
	TInt Internalise(RFile& aFile);

protected:
	CInstalledGameList(TInt aMaxGames);
	void ConstuctL();
	TInt	iGames;
	TInt	iMaxGames;
	TInt*	iGameIdxs;
};

class CMameEngine;

class MGameWatcher
{
public:
	virtual void NotifyGameFinished() = 0;
};


class CGameRunner : public CActive
{
public:
	static CGameRunner* NewL(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine);
	~CGameRunner();

	void RunGameL(TInt aGame);
	void TerminateGame();

protected:
	CGameRunner(MGameWatcher& aGameWatcher, CMameEngine& aMameEngine);
	void ConstructL();

	virtual void RunL();
	virtual void DoCancel();

	void DoRunGameL();
	static TInt ThreadMameGame(TAny* aData);
	void ShutdownGame();
	
	MGameWatcher&		iGameWatcher;
	RThread				iThread;
	TInt				iGame;
	TInt				iGameCounter;
	CMameEngine&		iMameEngine;
};




class CMameEngine : public CBase
{
public:
	static CMameEngine* NewL(RFs& aFs);
	~CMameEngine();

	TGameOptions				GameOptions() {return iGameOptions;}
	void						SetGameOptions(const TGameOptions& aGameOptions);
	void						SetWindowGroupId(TInt aWindowGroupId);
	void						ShutdownGame();
	const struct GameDriver**	GameDrivers();
	const char*					CpuName(TInt aCpuType);
	TInt						GameNo(const TDesC& aGameName);
	const char*					GameName(TInt aGameNo);
	const char*					GameShortName(TInt aGameNo);
	TBool						IsGameCloneOf(const struct GameDriver* aGame);
	void						ForceGameExit();
	void						ResumeGame();
	const CInstalledGameList&	InstalledGameList();
	void						SetInstalledGameList(const CInstalledGameList& aInstalledGameList);
	TInt						NoOfGames(){return iNoOfGames;}

	void CreateZipList();
	void DestroyZipList();

protected:
	CMameEngine(RFs& aFs);
	void ConstructL();

	void LoadOptions();
	TInt SaveOptions();
	void DefaultOptions();

	void LoadInstalledGameList();
	TInt SaveInstalledGameList();

	void CalcNoOfGames();

	friend class CGameRunner;
	void RunGameL(TInt aGame);
	void KillRunningStdoutConsole();

	TGameOptions	iGameOptions;

	RFs&				iFs;
	TInt				iAppWindowGroupId;
	CInstalledGameList*	iInstalledGameList;
	TInt				iNoOfGames;

};


#endif			/* __EMAME_ENGINE_H */
