/*******************************************************************
 *
 *	File:		Audio.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "audio.h"
#include "polledas.h"

#include <f32file.h>
#include <e32svr.h>
#include <string.h>




//#define	MAX_SOUND_BUFFER (133 * 1000)
TInt		sSamplesPerFrame = 133;


/*
 * In ER5 we can directly interface with the sound driver. In ER6 the preferred
 * way is to interace with the MediaServer. Apart from not having figured out
 * the MediaServer API, I suspect it will require client-server interaction.
 * Most likely that will be killing for EMame's performance. 
 *
 * I have taken yet another shortcut...
 * I noticed that in ER6 the sound driver's include files are not longer present
 * in \epoc32\include, however, the WINS sound driver binaries (esound.pdd and 
 * esdrv.ldd) are still there. I simply copied the ER5 include files over
 * to ER6, with the one change that in ER6 RDevSound should derive from 
 * RBusLogicalChannel rather than RLogicalChannel. If I then explicitly load 
 * the 2 sound drivers, sound works just fine in ER6/WINS. I haven't got a clue
 * on whether this will work on target.
 */


/*******************************************
 *
 * CGameAudio
 *
 *******************************************/



CGameAudio::CGameAudio(TBool aStereo) : iStereo(aStereo)
{
}


CGameAudio* CGameAudio::NewL(TBool aStereo)
{
	CGameAudio*		self = new(ELeave) CGameAudio(aStereo);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();		// self
	return self;
}


CGameAudio::~CGameAudio()
{
	Destruct();
}

void CGameAudio::Destruct()
{
	delete iMdaAudioOutputStream;
	iMdaAudioOutputStream = NULL;

	for (TInt i=0 ; i<KMaxSoundBuffers ; i++)
	{
		delete[] iSoundBuffers[i];
		iSoundBuffers[i] = NULL;
	}

}

const TInt KBufferedSamples = 8000/2;

void CGameAudio::ConstructL()
{
	CMdaServer* server = CMdaServer::NewL();

	iMdaAudioOutputStream = CMdaAudioOutputStream::NewL(*this, server);
	iMdaAudioOutputStream->Open(&iMdaAudioDataSettings);//callback calls QueueIdle function

	WaitForOpenToCompleteL();

	iMdaAudioOutputStream->SetAudioPropertiesL(TMdaAudioDataSettings::ESampleRate8000Hz, TMdaAudioDataSettings::EChannelsMono);
	for (TInt i=0 ; i<KMaxSoundBuffers ; i++)
		iSoundBuffers[i] = new TUint16[KBufferedSamples];
	iCurrentPosition = iSoundBuffers[0];
}


TInt CGameAudio::Write(TInt16* aBuffer, TInt aSize)
{
	ASSERT(((iFrameCount+1)*sSamplesPerFrame)< (KBufferedSamples));
	
	TInt	size = aSize;
	while(size--)
	{
		*iCurrentPosition++ = *aBuffer++;
		aBuffer++;		// we're mono
	}
	iFrameCount++;

	if (iFrameCount == 6)
	{
		iHasCopied = EFalse;
		TPtrC8		ptr((const TText8*) iSoundBuffers[iCurrentBuffer], iFrameCount*sSamplesPerFrame*2);
		iMdaAudioOutputStream->WriteL(ptr);
		User::After(0);

		iFrameCount = 0;
		if (++iCurrentBuffer == KMaxSoundBuffers)
			iCurrentBuffer = 0;
		iCurrentPosition = iSoundBuffers[iCurrentBuffer];
	}
	return aSize;
}


#if 0
TInt CGameAudio::Write(TInt16* aBuffer, TInt aSize)
{
	TInt	size = aSize;
	iCurrentPosition = iSoundBuffer0;
	while(size--)
	{
		*iCurrentPosition++ = *aBuffer++;
		aBuffer++;		// we're mono
	}
	iFrameCount++;

	TPtrC8		ptr((const TText8*) (iSoundBuffer0), aSize*2);
	iMdaAudioOutputStream->WriteL(ptr);
	if (iFrameCount == 6)
	{
		User::After(0);
		iFrameCount = 0;
	}

	return aSize;
}
#endif

void CGameAudio::WaitForCopyToCompleteL()
{
	while (!iHasCopied)
		CPolledActiveScheduler::Instance()->Schedule();
}

void CGameAudio::WaitForOpenToCompleteL()
{
	TInt	count = 20;		// 5 seconds
	TInt	waitPeriod = 250 * 1000;
	while (!iIsOpen && --count)
	{
		User::After(waitPeriod);
		CPolledActiveScheduler::Instance()->Schedule();
	}
	if (!iIsOpen)
		User::LeaveIfError(KErrNotSupported);
}

void CGameAudio::MaoscOpenComplete(TInt aError)
{
	if (aError)
		RDebug::Print(_L("CGameAudio::MaoscOpenComplete, error=%d"), aError);
	iIsOpen = ETrue;
}

void CGameAudio::MaoscBufferCopied(TInt aError, const TDesC8& /*aBuffer*/)
{
	iHasCopied = ETrue;
//	if (aError)
//		RDebug::Print(_L("CGameAudio::MaoscBufferCopied, error=%d"), aError);
}

void CGameAudio::MaoscPlayComplete(TInt aError)
{
//	if (aError)
//		RDebug::Print(_L("CGameAudio::MaoscPlayComplete, error=%d"), aError);
}
