/*******************************************************************
 *
 *	File:		EMame.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2001, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

extern "C" {
#include "osdepend.h"
#include "mame.h"
#include "driver.h"
}

#undef NULL

#include <stdio.h>
#include <unistd.h>
#include <e32std.h>
#include <f32file.h>
#include <e32svr.h>
#include <estlib.h>
#include <eikdll.h>
#include <w32std.h>


#include "emame.h"
#include "audio.h"		// for sample rate
#include "EMameGlobal.h"

static TGameOptions		sGameOptions;

const TGameOptions& GameOptions()
{
	return sGameOptions;
}



#define CPU_TYPE(type) 	(type & ~CPU_FLAGS_MASK)
#define CPU_AUDIO(type)	(type & CPU_AUDIO_CPU)

void CheckAudioSupport(TInt aGame)
{
	if (sGameOptions.iSoundEnabled)
	{
		TBool		soundSupported = ETrue;

		const struct GameDriver*	game = *(drivers + aGame);
		for (TInt i=0 ; i<MAX_CPU && soundSupported; i++)
		{
			TInt	cpuType = game->drv->cpu[i].cpu_type;
			if (CPU_AUDIO(cpuType))
				soundSupported = EFalse;
		}

		if (!soundSupported)
		{
			User::InfoPrint(_L("Sound is not supported in this game"));
			sGameOptions.iSoundEnabled = EFalse;
			User::After(3 * 1000 * 1000);
		}
	}
}





void SetDefaultGamesOptions()
{
#ifdef _DEBUG
    options.mame_debug		= 0;
#else
    options.mame_debug		= 0;
#endif

	options.record			= 0;
	options.playback		= 0;
	options.cheat			= 0;
	options.gui_host		= 1;

//	options.samplerate			= 11025;
	options.samplerate			= 8000;
	options.use_samples			= 0;
	options.use_emulated_ym3812 = 0;

	options.norotate		= 0;
	options.ror				= 0;
	options.rol				= 0;
	options.flipx			= 0;
	options.flipy			= 0;

	options.translucency	= 0;
	options.antialias		= 0;
	options.use_artwork		= 0;

	options.beam   = (int)(1 /*playing_game_options.beam*/ * 0x00010000);
	if (options.beam < 0x00010000)    /* beam MUST be at least 1 pixel */
		options.beam = 0x00010000;
	if (options.beam > 0x00100000)    /* set max to 16 pixels */
		options.beam = 0x00100000;

	options.flicker   = (int)(100 /*playing_game_options.flicker*/ * 2.55);
	if (options.flicker < 0)
		options.flicker = 0;
	if (options.flicker > 255)
		options.flicker = 255;
}


void SendMameCloseEventL(TInt aAppWindowGroupId)
{
	TWsEvent	wsEvent;
	wsEvent.SetType(EMameShuttingDown);

	TAutoClose<RWsSession> wsSession;

	User::LeaveIfError(wsSession.iObj.Connect());
	wsSession.PushL();
	User::LeaveIfError(wsSession.iObj.SendEventToWindowGroup(aAppWindowGroupId, wsEvent));
	CleanupStack::PopAndDestroy();
}

void SendMameCloseEvent(TInt aAppWindowGroupId)
{
	TRAPD(err, SendMameCloseEventL(aAppWindowGroupId));
	if (err)
		User::InfoPrint(_L("EMAME: Please terminate the thread STDOUT in the task list"));
}

void RunGameL(TInt aGame)
{
	CheckAudioSupport(aGame);

	TInt err = run_game(aGame);
	
	if (err)
	{
			/*
			 * "Normal" game error usually because of roms not being present
			 */
		printf("Press any key to continue\n");
		getchar();
	}
}



void RunGame(TInt aGame, const TGameOptions& aGameOptions, TInt aAppWindowGroupId)
{
#ifdef __WINS__
	int argc=0;
	char** argv=0;
	char** envp=0;

	__crt0(argc,argv,envp);
#endif

	SetForcedExitFromFrontEnd(EFalse);
	SetDefaultGamesOptions();
	sGameOptions = aGameOptions;

	TRAPD(err, RunGameL(aGame));
	if (err)
	{
		Oops();
	}

	SendMameCloseEvent(aAppWindowGroupId);

#ifdef __WINS__
	fclose(stdin);
	fclose(stdout);
	fclose(stderr);
#endif
}


